import logging
import requests

logger = logging.getLogger(__name__)


class CommandNotFoundError(Exception):

    def __init__(self, command):
        self.command = command
        message = "Command %s not found on server" % command
        super(CommandNotFoundError, self).__init__(message)


class PHPListClient:

    def __init__(self, url, secret=None):
        self.url = url
        self.secret = secret
        self.session = requests.Session()

    def __getattr__(self, name):
        """try to automatically determine the API call"""
        if '_' in name:
            name_parts = name.split('_')
            command = ''.join([name_parts[0]] + [p.capitalize() for p in name_parts[1:]])
        else:
            command = name
        def apicall(**kwargs):
            return self.call_api(command, kwargs)
        return apicall

    def call_api(self, command, params):
        params['cmd'] = command
        if self.secret:
            params['secret'] = self.secret
        r = self.session.post(self.url, data=params)
        r.raise_for_status()
        output = r.json()
        logger.debug("output: %r", output)
        if output['status'] != 'success':
            if output['status'] == 'error':
                if output['data']['message'] == 'No function for provided [cmd] found!':
                    raise CommandNotFoundError(command)
                else:
                    raise ValueError(output['data']['message'])
            else:
                raise Exception("don't know how to handle: %r" % output)
        return output



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path


here = path.abspath(path.dirname(__file__))

#with open(path.join(here, 'README.md'), encoding='utf-8') as f:
#    long_description = f.read()
def read_md(filename):
    import pypandoc
    return pypandoc.convert('README.md', 'rst')

setup(
    name='phplist-client',
    version='1.0.1',

    description='Python client for the phplist REST API',
    long_description=read_md('README.md'),

    # The project's main homepage.
    url='https://github.com/tubaman/phplist-restapi-python',

    author='Ryan Nowakowski',
    author_email='tubaman@fattuba.com',

    # Choose your license
    license='Apache 2.0',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='phplist rest client',

    py_modules=["phplist"],
    install_requires=['requests', 'pypandoc'],
    setup_requires=['pypandoc'],
    extras_require={
        'dev': ['ipython'],
        'test': ['nose'],
    },
)
